@@ -3,6 +3,7 @@
 
 #include "gdata-service.h"
 #include "gdata-parsable.h"
+#include "gdata-batch-operation.h"
 #include "gdata-enums.h"
 
 /* enumerations from "./gdata/gdata-service.h" */
@@ -19,6 +20,7 @@ gdata_operation_type_get_type (void)
       { GDATA_OPERATION_DOWNLOAD, "GDATA_OPERATION_DOWNLOAD", "download" },
       { GDATA_OPERATION_UPLOAD, "GDATA_OPERATION_UPLOAD", "upload" },
       { GDATA_OPERATION_AUTHENTICATION, "GDATA_OPERATION_AUTHENTICATION", "authentication" },
+      { GDATA_OPERATION_BATCH, "GDATA_OPERATION_BATCH", "batch" },
       { 0, NULL, NULL }
     };
     etype = g_enum_register_static ("GDataOperationType", values);
@@ -41,6 +43,7 @@ gdata_service_error_get_type (void)
       { GDATA_SERVICE_ERROR_BAD_QUERY_PARAMETER, "GDATA_SERVICE_ERROR_BAD_QUERY_PARAMETER", "bad-query-parameter" },
       { GDATA_SERVICE_ERROR_NETWORK_ERROR, "GDATA_SERVICE_ERROR_NETWORK_ERROR", "network-error" },
       { GDATA_SERVICE_ERROR_PROXY_ERROR, "GDATA_SERVICE_ERROR_PROXY_ERROR", "proxy-error" },
+      { GDATA_SERVICE_ERROR_WITH_BATCH_OPERATION, "GDATA_SERVICE_ERROR_WITH_BATCH_OPERATION", "with-batch-operation" },
       { 0, NULL, NULL }
     };
     etype = g_enum_register_static ("GDataServiceError", values);
@@ -60,6 +63,7 @@ gdata_authentication_error_get_type (void)
       { GDATA_AUTHENTICATION_ERROR_ACCOUNT_DELETED, "GDATA_AUTHENTICATION_ERROR_ACCOUNT_DELETED", "account-deleted" },
       { GDATA_AUTHENTICATION_ERROR_ACCOUNT_DISABLED, "GDATA_AUTHENTICATION_ERROR_ACCOUNT_DISABLED", "account-disabled" },
       { GDATA_AUTHENTICATION_ERROR_SERVICE_DISABLED, "GDATA_AUTHENTICATION_ERROR_SERVICE_DISABLED", "service-disabled" },
+      { GDATA_AUTHENTICATION_ERROR_ACCOUNT_MIGRATED, "GDATA_AUTHENTICATION_ERROR_ACCOUNT_MIGRATED", "account-migrated" },
       { 0, NULL, NULL }
     };
     etype = g_enum_register_static ("GDataAuthenticationError", values);
@@ -83,5 +87,23 @@ gdata_parser_error_get_type (void)
   return etype;
 }
 
+/* enumerations from "./gdata/gdata-batch-operation.h" */
+GType
+gdata_batch_operation_type_get_type (void)
+{
+  static GType etype = 0;
+  if (etype == 0) {
+    static const GEnumValue values[] = {
+      { GDATA_BATCH_OPERATION_QUERY, "GDATA_BATCH_OPERATION_QUERY", "query" },
+      { GDATA_BATCH_OPERATION_INSERTION, "GDATA_BATCH_OPERATION_INSERTION", "insertion" },
+      { GDATA_BATCH_OPERATION_UPDATE, "GDATA_BATCH_OPERATION_UPDATE", "update" },
+      { GDATA_BATCH_OPERATION_DELETION, "GDATA_BATCH_OPERATION_DELETION", "deletion" },
+      { 0, NULL, NULL }
+    };
+    etype = g_enum_register_static ("GDataBatchOperationType", values);
+  }
+  return etype;
+}
+
 
 
