@@ -30,6 +30,53 @@
  * For more information on the custom GData query parameters supported by #GDataDocumentsQuery, see the <ulink type="http"
  * url="http://code.google.com/apis/documents/docs/2.0/reference.html#Parameters">online documentation</ulink>.
  *
+ * <example>
+ * 	<title>Querying for Documents</title>
+ * 	<programlisting>
+ *	GDataDocumentsService *service;
+ *	GDataDocumentsQuery *query;
+ *	GDataFeed *feed;
+ *	GTimeVal current_time;
+ *	GList *i;
+ *	GError *error = NULL;
+ *
+ *	/<!-- -->* Create a service *<!-- -->/
+ *	service = create_documents_service ();
+ *
+ *	/<!-- -->* Create the query to use. We're going to query for the last 10 documents modified by example@gmail.com in the past week, including
+ *	 * deleted documents. *<!-- -->/
+ *	query = gdata_documents_query_new_with_limits (NULL, 0, 10);
+ *
+ *	gdata_documents_query_add_collaborator (query, "example@gmail.com");
+ *	gdata_documents_query_set_show_deleted (query, TRUE);
+ *
+ *	g_get_current_time (&current_time);
+ *	gdata_query_set_updated_min (GDATA_QUERY (query), current_time.tv_sec - 7 * 24 * 60 * 60);
+ *	gdata_query_set_updated_max (GDATA_QUERY (query), current_time.tv_sec);
+ *
+ *	/<!-- -->* Execute the query *<!-- -->/
+ *	feed = gdata_documents_service_query_documents (service, query, NULL, NULL, NULL, &error);
+ *
+ *	g_object_unref (query);
+ *	g_object_unref (service);
+ *
+ *	if (error != NULL) {
+ *		g_error ("Error querying for documents: %s", error->message);
+ *		g_error_free (error);
+ *		return;
+ *	}
+ *
+ *	/<!-- -->* Iterate through the returned documents and do something with them *<!-- -->/
+ *	for (i = gdata_feed_get_entries (feed); i != NULL; i = i->next) {
+ *		GDataDocumentsDocument *document = GDATA_DOCUMENTS_DOCUMENT (i->data);
+ *
+ *		/<!-- -->* Do something with the document here, such as insert it into a UI *<!-- -->/
+ *	}
+ *
+ *	g_object_unref (feed);
+ * 	</programlisting>
+ * </example>
+ *
  * Since: 0.4.0
  **/
 
@@ -47,6 +94,7 @@
 #include <gdata/services/documents/gdata-documents-text.h>
 #include <gdata/services/documents/gdata-documents-folder.h>
 
+static void gdata_documents_query_dispose (GObject *object);
 static void gdata_documents_query_finalize (GObject *object);
 static void gdata_documents_query_get_property (GObject *object, guint property_id, GValue *value, GParamSpec *pspec);
 static void gdata_documents_query_set_property (GObject *object, guint property_id, const GValue *value, GParamSpec *pspec);
@@ -71,7 +119,6 @@ enum {
 };
 
 G_DEFINE_TYPE (GDataDocumentsQuery, gdata_documents_query, GDATA_TYPE_QUERY)
-#define GDATA_DOCUMENTS_QUERY_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), GDATA_TYPE_DOCUMENTS_QUERY, GDataDocumentsQueryPrivate))
 
 static void
 gdata_documents_query_class_init (GDataDocumentsQueryClass *klass)
@@ -83,6 +130,7 @@ gdata_documents_query_class_init (GDataDocumentsQueryClass *klass)
 
 	gobject_class->get_property = gdata_documents_query_get_property;
 	gobject_class->set_property = gdata_documents_query_set_property;
+	gobject_class->dispose = gdata_documents_query_dispose;
 	gobject_class->finalize = gdata_documents_query_finalize;
 
 	query_class->get_query_uri = get_query_uri;
@@ -95,10 +143,10 @@ gdata_documents_query_class_init (GDataDocumentsQueryClass *klass)
 	 * Since: 0.4.0
 	 **/
 	g_object_class_install_property (gobject_class, PROP_SHOW_DELETED,
-				g_param_spec_boolean ("show-deleted",
-					"Show deleted?", "A shortcut to request all documents that have been deleted.",
-					FALSE,
-					G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+	                                 g_param_spec_boolean ("show-deleted",
+	                                                       "Show deleted?", "A shortcut to request all documents that have been deleted.",
+	                                                       FALSE,
+	                                                       G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
 	/**
 	 * GDataDocumentsQuery:show-folders:
@@ -108,10 +156,10 @@ gdata_documents_query_class_init (GDataDocumentsQueryClass *klass)
 	 * Since: 0.4.0
 	 **/
 	g_object_class_install_property (gobject_class, PROP_SHOW_FOLDERS,
-				g_param_spec_boolean ("show-folders",
-					"Show folders?", "Specifies if the request also returns folders.",
-					FALSE,
-					G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+	                                 g_param_spec_boolean ("show-folders",
+	                                                       "Show folders?", "Specifies if the request also returns folders.",
+	                                                       FALSE,
+	                                                       G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
 	/**
 	 * GDataDocumentsQuery:exact-title:
@@ -121,10 +169,10 @@ gdata_documents_query_class_init (GDataDocumentsQueryClass *klass)
 	 * Since: 0.4.0
 	 **/
 	g_object_class_install_property (gobject_class, PROP_EXACT_TITLE,
-				g_param_spec_boolean ("exact-title",
-					"Exact title?", "Specifies whether the query should search for an exact title match.",
-					FALSE,
-					G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+	                                 g_param_spec_boolean ("exact-title",
+	                                                       "Exact title?", "Specifies whether the query should search for an exact title match.",
+	                                                       FALSE,
+	                                                       G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
 	/**
 	 * GDataDocumentsQuery:folder-id:
@@ -134,10 +182,10 @@ gdata_documents_query_class_init (GDataDocumentsQueryClass *klass)
 	 * Since: 0.4.0
 	 **/
 	g_object_class_install_property (gobject_class, PROP_FOLDER_ID,
-				g_param_spec_string ("folder-id",
-					"Folder ID", "Specifies the ID of the folder in which to search.",
-					NULL,
-					G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+	                                 g_param_spec_string ("folder-id",
+	                                                      "Folder ID", "Specifies the ID of the folder in which to search.",
+	                                                      NULL,
+	                                                      G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
 	/**
 	 * GDataDocumentsQuery:title:
@@ -148,10 +196,10 @@ gdata_documents_query_class_init (GDataDocumentsQueryClass *klass)
 	 * Since: 0.4.0
 	 **/
 	g_object_class_install_property (gobject_class, PROP_TITLE,
-				g_param_spec_string ("title",
-					"Title", "A title (or title fragment) to be searched for.",
-					NULL,
-					G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+	                                 g_param_spec_string ("title",
+	                                                      "Title", "A title (or title fragment) to be searched for.",
+	                                                      NULL,
+	                                                      G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 }
 
 static void
@@ -160,10 +208,40 @@ gdata_documents_query_init (GDataDocumentsQuery *self)
 	self->priv = G_TYPE_INSTANCE_GET_PRIVATE (self, GDATA_TYPE_DOCUMENTS_QUERY, GDataDocumentsQueryPrivate);
 }
 
+static void
+gdata_documents_query_dispose (GObject *object)
+{
+	GList *i;
+	GDataDocumentsQueryPrivate *priv = GDATA_DOCUMENTS_QUERY (object)->priv;
+
+	for (i = priv->collaborator_addresses; i != NULL; i = i->next)
+		g_object_unref (i->data);
+	g_list_free (priv->collaborator_addresses);
+	priv->collaborator_addresses = NULL;
+
+	for (i = priv->reader_addresses; i != NULL; i = i->next)
+		g_object_unref (i->data);
+	g_list_free (priv->reader_addresses);
+	priv->reader_addresses = NULL;
+
+	G_OBJECT_CLASS (gdata_documents_query_parent_class)->dispose (object);
+}
+
+static void
+gdata_documents_query_finalize (GObject *object)
+{
+	GDataDocumentsQueryPrivate *priv = GDATA_DOCUMENTS_QUERY (object)->priv;
+
+	g_free (priv->folder_id);
+	g_free (priv->title);
+
+	G_OBJECT_CLASS (gdata_documents_query_parent_class)->finalize (object);
+}
+
 static void
 gdata_documents_query_get_property (GObject *object, guint property_id, GValue *value, GParamSpec *pspec)
 {
-	GDataDocumentsQueryPrivate *priv = GDATA_DOCUMENTS_QUERY_GET_PRIVATE (object);
+	GDataDocumentsQueryPrivate *priv = GDATA_DOCUMENTS_QUERY (object)->priv;
 
 	switch (property_id) {
 		case PROP_SHOW_DELETED:
@@ -216,26 +294,14 @@ gdata_documents_query_set_property (GObject *object, guint property_id, const GV
 	}
 }
 
-static void
-gdata_documents_query_finalize (GObject *object)
-{
-	GDataDocumentsQueryPrivate *priv = GDATA_DOCUMENTS_QUERY_GET_PRIVATE (object);
-
-	g_free (priv->folder_id);
-	g_free (priv->title);
-
-	G_OBJECT_CLASS (gdata_documents_query_parent_class)->finalize (object);
-}
-
 static void
 get_query_uri (GDataQuery *self, const gchar *feed_uri, GString *query_uri, gboolean *params_started)
 {
 	GDataDocumentsQueryPrivate *priv = GDATA_DOCUMENTS_QUERY (self)->priv;
-	const gchar *entry_id = gdata_query_get_entry_id (self);
 
 	#define APPEND_SEP g_string_append_c (query_uri, (*params_started == FALSE) ? '?' : '&'); *params_started = TRUE;
 
-	if (entry_id == NULL && priv->folder_id != NULL) {
+	if (priv->folder_id != NULL) {
 		g_string_append (query_uri, "/folder%3A");
 		g_string_append_uri_escaped (query_uri, priv->folder_id, NULL, FALSE);
 	}
@@ -243,33 +309,29 @@ get_query_uri (GDataQuery *self, const gchar *feed_uri, GString *query_uri, gboo
 	/* Chain up to the parent class */
 	GDATA_QUERY_CLASS (gdata_documents_query_parent_class)->get_query_uri (self, feed_uri, query_uri, params_started);
 
-	/* Return if the entry ID has been set, since that's handled in the parent class' get_query_uri() function */
-	if (entry_id != NULL)
-		return;
-
 	if  (priv->collaborator_addresses != NULL) {
-		GList *collaborator_address;
+		GList *address;
 		APPEND_SEP
-		collaborator_address = priv->collaborator_addresses;
+		address = priv->collaborator_addresses;
 
 		g_string_append (query_uri, "writer=");
-		g_string_append_uri_escaped (query_uri, gdata_gd_email_address_get_address (collaborator_address->data), NULL, FALSE);
-		for (collaborator_address = collaborator_address->next; collaborator_address != NULL; collaborator_address = collaborator_address->next) {
+		g_string_append_uri_escaped (query_uri, gdata_gd_email_address_get_address (address->data), NULL, FALSE);
+		for (address = address->next; address != NULL; address = address->next) {
 			g_string_append_c (query_uri, ';');
-			g_string_append_uri_escaped (query_uri, gdata_gd_email_address_get_address (collaborator_address->data), NULL, FALSE);
+			g_string_append_uri_escaped (query_uri, gdata_gd_email_address_get_address (address->data), NULL, FALSE);
 		}
 	}
 
 	if  (priv->reader_addresses != NULL) {
-		GList *reader_address;
+		GList *address;
 		APPEND_SEP
-		reader_address = priv->reader_addresses;
+		address = priv->reader_addresses;
 
 		g_string_append (query_uri, "reader=");
-		g_string_append_uri_escaped (query_uri, gdata_gd_email_address_get_address (reader_address->data), NULL, FALSE);
-		for (reader_address = reader_address->next; reader_address != NULL; reader_address = reader_address->next) {
+		g_string_append_uri_escaped (query_uri, gdata_gd_email_address_get_address (address->data), NULL, FALSE);
+		for (address = address->next; address != NULL; address = address->next) {
 			g_string_append_c (query_uri, ';');
-			g_string_append_uri_escaped (query_uri, gdata_gd_email_address_get_address (reader_address->data), NULL, FALSE);
+			g_string_append_uri_escaped (query_uri, gdata_gd_email_address_get_address (address->data), NULL, FALSE);
 		}
 	}
 
@@ -295,7 +357,7 @@ get_query_uri (GDataQuery *self, const gchar *feed_uri, GString *query_uri, gboo
 
 /**
  * gdata_documents_query_new:
- * @q: a query string
+ * @q: (allow-none): a query string, or %NULL
  *
  * Creates a new #GDataDocumentsQuery with its #GDataQuery:q property set to @q.
  *
@@ -311,9 +373,9 @@ gdata_documents_query_new (const gchar *q)
 
 /**
  * gdata_documents_query_new_with_limits:
- * @q: a query string
- * @start_index: a one-based start index for the results
- * @max_results: the maximum number of results to return
+ * @q: (allow-none): a query string, or %NULL
+ * @start_index: a one-based start index for the results, or <code class="literal">0</code>
+ * @max_results: the maximum number of results to return, or <code class="literal">0</code>
  *
  * Creates a new #GDataDocumentsQuery with its #GDataQuery:q property set to @q, and the limits @start_index and @max_results
  * applied.
@@ -323,13 +385,13 @@ gdata_documents_query_new (const gchar *q)
  * Since: 0.4.0
  **/
 GDataDocumentsQuery *
-gdata_documents_query_new_with_limits (const gchar *q, gint start_index, gint max_results)
+gdata_documents_query_new_with_limits (const gchar *q, guint start_index, guint max_results)
 {
 	return g_object_new (GDATA_TYPE_DOCUMENTS_QUERY,
-			     "q", q,
-			     "start-index", start_index,
-			     "max-results", max_results,
-			     NULL);
+	                     "q", q,
+	                     "start-index", start_index,
+	                     "max-results", max_results,
+	                     NULL);
 }
 
 /**
@@ -426,7 +488,7 @@ gdata_documents_query_get_folder_id (GDataDocumentsQuery *self)
 /**
  * gdata_documents_query_set_folder_id:
  * @self: a #GDataDocumentsQuery
- * @folder_id: the ID of the folder to be queried, or %NULL
+ * @folder_id: (allow-none): the ID of the folder to be queried, or %NULL
  *
  * Sets the #GDataDocumentsQuery:folder-id property to @folder_id.
  *
@@ -484,7 +546,7 @@ gdata_documents_query_get_exact_title (GDataDocumentsQuery *self)
 /**
  * gdata_documents_query_set_title:
  * @self: a #GDataDocumentsQuery
- * @title: the title (or title fragment) to query for, or %NULL
+ * @title: (allow-none): the title (or title fragment) to query for, or %NULL
  * @exact_title: %TRUE if the query should match the exact @title, %FALSE otherwise
  *
  * Sets the #GDataDocumentsQuery:title property to @title.
@@ -517,7 +579,8 @@ gdata_documents_query_set_title (GDataDocumentsQuery *self, const gchar *title,
  *
  * Gets a list of #GDataGDEmailAddress<!-- -->es of the document collaborators whose documents will be queried.
  *
- * Return value: a list of #GDataGDEmailAddress<!-- -->es of the collaborators concerned by the query, or %NULL
+ * Return value: (element-type GData.GDEmailAddress) (transfer none): a list of #GDataGDEmailAddress<!-- -->es of the collaborators concerned by the
+ * query, or %NULL
  *
  * Since: 0.4.0
  **/
@@ -534,7 +597,8 @@ gdata_documents_query_get_collaborator_addresses (GDataDocumentsQuery *self)
  *
  * Gets a list of #GDataGDEmailAddress<!-- -->es of the document readers whose documents will be queried.
  *
- * Return value: a list of #GDataGDEmailAddress<!-- -->es of the readers concerned by the query, or %NULL
+ * Return value: (element-type GData.GDEmailAddress) (transfer none): a list of #GDataGDEmailAddress<!-- -->es of the readers concerned by the query,
+ * or %NULL
  *
  * Since: 0.4.0
  **/
