@@ -1,7 +1,7 @@
 /* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 8; tab-width: 8 -*- */
 /*
  * GData Client
- * Copyright (C) Philip Withnall 2009 <philip@tecnocode.co.uk>
+ * Copyright (C) Philip Withnall 2009–2010 <philip@tecnocode.co.uk>
  *
  * GData Client is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -28,6 +28,8 @@
  *
  * For more information on the custom GData query parameters supported by #GDataYouTubeQuery, see the <ulink type="http"
  * url="http://code.google.com/apis/youtube/2.0/reference.html#Custom_parameters">online documentation</ulink>.
+ *
+ * Since: 0.3.0
  **/
 
 #include <config.h>
@@ -75,7 +77,6 @@ enum {
 };
 
 G_DEFINE_TYPE (GDataYouTubeQuery, gdata_youtube_query, GDATA_TYPE_QUERY)
-#define GDATA_YOUTUBE_QUERY_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), GDATA_TYPE_YOUTUBE_QUERY, GDataYouTubeQueryPrivate))
 
 static void
 gdata_youtube_query_class_init (GDataYouTubeQueryClass *klass)
@@ -100,20 +101,21 @@ gdata_youtube_query_class_init (GDataYouTubeQueryClass *klass)
 	 * Since: 0.3.0
 	 **/
 	g_object_class_install_property (gobject_class, PROP_FORMAT,
-				g_param_spec_enum ("format",
-					"Format", "Specifies that videos must be available in a particular video format.",
-					GDATA_TYPE_YOUTUBE_FORMAT, GDATA_YOUTUBE_FORMAT_UNKNOWN,
-					G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+	                                 g_param_spec_enum ("format",
+	                                                    "Format", "Specifies that videos must be available in a particular video format.",
+	                                                    GDATA_TYPE_YOUTUBE_FORMAT, GDATA_YOUTUBE_FORMAT_UNKNOWN,
+	                                                    G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
 	/**
 	 * GDataYouTubeQuery:latitude:
 	 *
 	 * The latitude of a particular location of which videos should be found. This should be used in conjunction with
 	 * #GDataYouTubeQuery:longitude; if either property is outside the valid range, neither will be used. Valid latitudes
-	 * are between %-90 and %90 degrees; any values of this property outside that range will unset the property in the
-	 * query URI.
+	 * are between <code class="literal">-90</code> and <code class="literal">90</code>0 degrees; any values of this property outside that range
+	 * will unset the property in the query URI.
 	 *
-	 * If #GDataYouTubeQuery:location-radius is a non-%0 value, this will define a circle from which videos should be found.
+	 * If #GDataYouTubeQuery:location-radius is a non-<code class="literal">0</code> value, this will define a circle from which videos should be
+	 * found.
 	 *
 	 * If #GDataYouTubeQuery:has-location is %TRUE, only videos which are associated with specific coordinates in the search
 	 * circle will be returned. Otherwise, videos which have a relevant descriptive address (but no specific coordinates) will
@@ -125,28 +127,28 @@ gdata_youtube_query_class_init (GDataYouTubeQueryClass *klass)
 	 * Since: 0.3.0
 	 **/
 	g_object_class_install_property (gobject_class, PROP_LATITUDE,
-				g_param_spec_double ("latitude",
-					"Latitude", "The latitude of a particular location of which videos should be found.",
-					-G_MAXDOUBLE, G_MAXDOUBLE, 0.0,
-					G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+	                                 g_param_spec_double ("latitude",
+	                                                      "Latitude", "The latitude of a particular location of which videos should be found.",
+	                                                      -G_MAXDOUBLE, G_MAXDOUBLE, 0.0,
+	                                                      G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
 	/**
 	 * GDataYouTubeQuery:longitude:
 	 *
 	 * The longitude of a particular location of which videos should be found. This should be used in conjunction with
 	 * #GDataYouTubeQuery:latitude; if either property is outside the valid range, neither will be used. Valid longitudes
-	 * are between %-180 and %180 degrees; any values of this property outside that range will unset the property in the
-	 * query URI.
+	 * are between <code class="literal">-180</code> and <code class="literal">180</code> degrees; any values of this property outside that
+	 * range will unset the property in the query URI.
 	 *
 	 * For more information, see the documentation for #GDataYouTubeQuery:latitude.
 	 *
 	 * Since: 0.3.0
 	 **/
 	g_object_class_install_property (gobject_class, PROP_LONGITUDE,
-				g_param_spec_double ("longitude",
-					"Longitude", "The longitude of a particular location of which videos should be found.",
-					-G_MAXDOUBLE, G_MAXDOUBLE, 0.0,
-					G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+	                                 g_param_spec_double ("longitude",
+	                                                      "Longitude", "The longitude of a particular location of which videos should be found.",
+	                                                      -G_MAXDOUBLE, G_MAXDOUBLE, 0.0,
+	                                                      G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
 	/**
 	 * GDataYouTubeQuery:location-radius:
@@ -154,17 +156,17 @@ gdata_youtube_query_class_init (GDataYouTubeQueryClass *klass)
 	 * The radius, in metres, of a circle from within which videos should be returned. The circle is centred on the latitude and
 	 * longitude given in #GDataYouTubeQuery:latitude and #GDataYouTubeQuery:longitude.
 	 *
-	 * Set this property to %0 to search for specific coordinates, rather than within a given radius.
+	 * Set this property to <code class="literal">0</code> to search for specific coordinates, rather than within a given radius.
 	 *
 	 * For more information, see the documentation for #GDataYouTubeQuery:latitude.
 	 *
 	 * Since: 0.3.0
 	 **/
 	g_object_class_install_property (gobject_class, PROP_LOCATION_RADIUS,
-				g_param_spec_double ("location-radius",
-					"Location radius", "The radius, in metres, of a circle from within which videos should be returned.",
-					0.0, G_MAXDOUBLE, 0.0,
-					G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+	                                 g_param_spec_double ("location-radius",
+	                                                      "Location radius", "The radius, in metres, of a circle to search within.",
+	                                                      0.0, G_MAXDOUBLE, 0.0,
+	                                                      G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
 	/**
 	 * GDataYouTubeQuery:has-location:
@@ -179,10 +181,10 @@ gdata_youtube_query_class_init (GDataYouTubeQueryClass *klass)
 	 * Since: 0.3.0
 	 **/
 	g_object_class_install_property (gobject_class, PROP_HAS_LOCATION,
-				g_param_spec_boolean ("has-location",
-					"Has location?", "Whether to restrict search results to videos with specific coordinates associated with them.",
-					FALSE,
-					G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+	                                 g_param_spec_boolean ("has-location",
+	                                                       "Has location?", "Whether to restrict results to videos with specific coordinates.",
+	                                                       FALSE,
+	                                                       G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
 	/**
 	 * GDataYouTubeQuery:language:
@@ -197,10 +199,10 @@ gdata_youtube_query_class_init (GDataYouTubeQueryClass *klass)
 	 * Since: 0.3.0
 	 **/
 	g_object_class_install_property (gobject_class, PROP_LANGUAGE,
-				g_param_spec_string ("language",
-					"Language", "Restricts the search to videos that have a title, description or keywords in a specified language.",
-					NULL,
-					G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+	                                 g_param_spec_string ("language",
+	                                                      "Language", "Restricts the search to videos described in the given language.",
+	                                                      NULL,
+	                                                      G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
 	/**
 	 * GDataYouTubeQuery:order-by:
@@ -218,10 +220,10 @@ gdata_youtube_query_class_init (GDataYouTubeQueryClass *klass)
 	 * Since: 0.3.0
 	 **/
 	g_object_class_install_property (gobject_class, PROP_ORDER_BY,
-				g_param_spec_string ("order-by",
-					"Order by", "Specifies the order of entries in a feed.",
-					NULL,
-					G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+	                                 g_param_spec_string ("order-by",
+	                                                      "Order by", "Specifies the order of entries in a feed.",
+	                                                      NULL,
+	                                                      G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
 	/**
 	 * GDataYouTubeQuery:restriction:
@@ -236,10 +238,10 @@ gdata_youtube_query_class_init (GDataYouTubeQueryClass *klass)
 	 * Since: 0.3.0
 	 **/
 	g_object_class_install_property (gobject_class, PROP_RESTRICTION,
-				g_param_spec_string ("restriction",
-					"Restriction", "The IP address that should be used to filter videos playable only in specific countries.",
-					NULL,
-					G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+	                                 g_param_spec_string ("restriction",
+	                                                      "Restriction", "The IP address to filter videos playable only in specific countries.",
+	                                                      NULL,
+	                                                      G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
 	/**
 	 * GDataYouTubeQuery:safe-search:
@@ -252,10 +254,10 @@ gdata_youtube_query_class_init (GDataYouTubeQueryClass *klass)
 	 * Since: 0.3.0
 	 **/
 	g_object_class_install_property (gobject_class, PROP_SAFE_SEARCH,
-				g_param_spec_enum ("safe-search",
-					"Safe search", "Whether the search results should include restricted content as well as standard content.",
-					GDATA_TYPE_YOUTUBE_SAFE_SEARCH, GDATA_YOUTUBE_SAFE_SEARCH_MODERATE,
-					G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+	                                 g_param_spec_enum ("safe-search",
+	                                                    "Safe search", "Whether the search results should include restricted content.",
+	                                                    GDATA_TYPE_YOUTUBE_SAFE_SEARCH, GDATA_YOUTUBE_SAFE_SEARCH_MODERATE,
+	                                                    G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
 	/**
 	 * GDataYouTubeQuery:sort-order:
@@ -265,10 +267,10 @@ gdata_youtube_query_class_init (GDataYouTubeQueryClass *klass)
 	 * Since: 0.3.0
 	 **/
 	g_object_class_install_property (gobject_class, PROP_SORT_ORDER,
-				g_param_spec_enum ("sort-order",
-					"Sort order", "Specifies the direction of sorting.",
-					GDATA_TYPE_YOUTUBE_SORT_ORDER, GDATA_YOUTUBE_SORT_NONE,
-					G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+	                                 g_param_spec_enum ("sort-order",
+	                                                    "Sort order", "Specifies the direction of sorting.",
+	                                                    GDATA_TYPE_YOUTUBE_SORT_ORDER, GDATA_YOUTUBE_SORT_NONE,
+	                                                    G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
 	/**
 	 * GDataYouTubeQuery:age:
@@ -279,10 +281,10 @@ gdata_youtube_query_class_init (GDataYouTubeQueryClass *klass)
 	 * Since: 0.3.0
 	 **/
 	g_object_class_install_property (gobject_class, PROP_AGE,
-				g_param_spec_enum ("age",
-					"Age", "Restricts the search to videos uploaded within the specified time period.",
-					GDATA_TYPE_YOUTUBE_AGE, GDATA_YOUTUBE_AGE_ALL_TIME,
-					G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+	                                 g_param_spec_enum ("age",
+	                                                    "Age", "Restricts the search to videos uploaded within the specified time period.",
+	                                                    GDATA_TYPE_YOUTUBE_AGE, GDATA_YOUTUBE_AGE_ALL_TIME,
+	                                                    G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
 	/**
 	 * GDataYouTubeQuery:uploader:
@@ -293,10 +295,10 @@ gdata_youtube_query_class_init (GDataYouTubeQueryClass *klass)
 	 * Since: 0.3.0
 	 **/
 	g_object_class_install_property (gobject_class, PROP_UPLOADER,
-				g_param_spec_enum ("uploader",
-					"Uploader", "Restricts the search to videos from the specified type of uploader.",
-					GDATA_TYPE_YOUTUBE_UPLOADER, GDATA_YOUTUBE_UPLOADER_ALL,
-					G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+	                                 g_param_spec_enum ("uploader",
+	                                                    "Uploader", "Restricts the search to videos from the specified type of uploader.",
+	                                                    GDATA_TYPE_YOUTUBE_UPLOADER, GDATA_YOUTUBE_UPLOADER_ALL,
+	                                                    G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 }
 
 static void
@@ -308,7 +310,7 @@ gdata_youtube_query_init (GDataYouTubeQuery *self)
 static void
 gdata_youtube_query_finalize (GObject *object)
 {
-	GDataYouTubeQueryPrivate *priv = GDATA_YOUTUBE_QUERY_GET_PRIVATE (object);
+	GDataYouTubeQueryPrivate *priv = GDATA_YOUTUBE_QUERY (object)->priv;
 
 	g_free (priv->language);
 	g_free (priv->order_by);
@@ -321,7 +323,7 @@ gdata_youtube_query_finalize (GObject *object)
 static void
 gdata_youtube_query_get_property (GObject *object, guint property_id, GValue *value, GParamSpec *pspec)
 {
-	GDataYouTubeQueryPrivate *priv = GDATA_YOUTUBE_QUERY_GET_PRIVATE (object);
+	GDataYouTubeQueryPrivate *priv = GDATA_YOUTUBE_QUERY (object)->priv;
 
 	switch (property_id) {
 		case PROP_FORMAT:
@@ -378,19 +380,15 @@ gdata_youtube_query_set_property (GObject *object, guint property_id, const GVal
 			break;
 		case PROP_LATITUDE:
 			self->priv->latitude = g_value_get_double (value);
-			g_object_notify (object, "latitude");
 			break;
 		case PROP_LONGITUDE:
 			self->priv->longitude = g_value_get_double (value);
-			g_object_notify (object, "longitude");
 			break;
 		case PROP_LOCATION_RADIUS:
 			self->priv->location_radius = g_value_get_double (value);
-			g_object_notify (object, "location-radius");
 			break;
 		case PROP_HAS_LOCATION:
 			self->priv->has_location = g_value_get_boolean (value);
-			g_object_notify (object, "has-location");
 			break;
 		case PROP_LANGUAGE:
 			gdata_youtube_query_set_language (self, g_value_get_string (value));
@@ -471,8 +469,8 @@ get_query_uri (GDataQuery *self, const gchar *feed_uri, GString *query_uri, gboo
 		gchar latitude[G_ASCII_DTOSTR_BUF_SIZE], longitude[G_ASCII_DTOSTR_BUF_SIZE];
 
 		g_string_append_printf (query_uri, (priv->has_location == TRUE) ? "&location=%s,%s!" : "&location=%s,%s",
-					g_ascii_dtostr (latitude, sizeof (latitude), priv->latitude),
-					g_ascii_dtostr (longitude, sizeof (longitude), priv->longitude));
+		                        g_ascii_dtostr (latitude, sizeof (latitude), priv->latitude),
+		                        g_ascii_dtostr (longitude, sizeof (longitude), priv->longitude));
 
 		if (priv->location_radius >= 0.0) {
 			gchar radius[G_ASCII_DTOSTR_BUF_SIZE];
@@ -512,7 +510,7 @@ get_query_uri (GDataQuery *self, const gchar *feed_uri, GString *query_uri, gboo
 
 /**
  * gdata_youtube_query_new:
- * @q: a query string
+ * @q: (allow-none): a query string, or %NULL
  *
  * Creates a new #GDataYouTubeQuery with its #GDataQuery:q property set to @q.
  *
@@ -558,15 +556,19 @@ gdata_youtube_query_set_format (GDataYouTubeQuery *self, GDataYouTubeFormat form
 	g_return_if_fail (GDATA_IS_YOUTUBE_QUERY (self));
 	self->priv->format = format;
 	g_object_notify (G_OBJECT (self), "format");
+
+	/* Our current ETag will no longer be relevant */
+	gdata_query_set_etag (GDATA_QUERY (self), NULL);
 }
 
 /**
  * gdata_youtube_query_get_location:
  * @self: a #GDataYouTubeQuery
- * @latitude: a location in which to return the latitude, or %NULL
- * @longitude: a location in which to return the longitude, or %NULL
- * @radius: a location in which to return the location radius, or %NULL
- * @has_location: a location in which to return %TRUE if the query is searching for videos with a specific location, %FALSE otherwise, or %NULL
+ * @latitude: (out caller-allocates) (allow-none): a location in which to return the latitude, or %NULL
+ * @longitude: (out caller-allocates) (allow-none): a location in which to return the longitude, or %NULL
+ * @radius: (out caller-allocates) (allow-none): a location in which to return the location radius, or %NULL
+ * @has_location: (out caller-allocates) (allow-none): a location in which to return %TRUE if the query is searching for videos with a specific
+ * location, %FALSE otherwise, or %NULL
  *
  * Gets the location-based properties of the #GDataYouTubeQuery<!-- -->: #GDataYouTubeQuery:latitude, #GDataYouTubeQuery:longitude,
  * #GDataYouTubeQuery:location-radius and #GDataYouTubeQuery:has-location.
@@ -593,7 +595,7 @@ gdata_youtube_query_get_location (GDataYouTubeQuery *self, gdouble *latitude, gd
  * @self: a #GDataYouTubeQuery
  * @latitude: the new latitude, or %G_MAXDOUBLE
  * @longitude: the new longitude, or %G_MAXDOUBLE
- * @radius: the new location radius, or %0
+ * @radius: the new location radius, or <code class="literal">0</code>
  * @has_location: %TRUE if the query is for videos with a specific location, %FALSE otherwise
  *
  * Sets the location-based properties of the #GDataYouTubeQuery<!-- -->: #GDataYouTubeQuery:latitude, #GDataYouTubeQuery:longitude,
@@ -611,10 +613,15 @@ gdata_youtube_query_set_location (GDataYouTubeQuery *self, gdouble latitude, gdo
 	self->priv->location_radius = radius;
 	self->priv->has_location = has_location;
 
+	g_object_freeze_notify (G_OBJECT (self));
 	g_object_notify (G_OBJECT (self), "latitude");
 	g_object_notify (G_OBJECT (self), "longitude");
 	g_object_notify (G_OBJECT (self), "location-radius");
 	g_object_notify (G_OBJECT (self), "has-location");
+	g_object_thaw_notify (G_OBJECT (self));
+
+	/* Our current ETag will no longer be relevant */
+	gdata_query_set_etag (GDATA_QUERY (self), NULL);
 }
 
 /**
@@ -637,7 +644,7 @@ gdata_youtube_query_get_language (GDataYouTubeQuery *self)
 /**
  * gdata_youtube_query_set_language:
  * @self: a #GDataYouTubeQuery
- * @language: a new language name, or %NULL
+ * @language: (allow-none): a new language name, or %NULL
  *
  * Sets the #GDataYouTubeQuery:language property of the #GDataYouTubeQuery to the new language, @language.
  *
@@ -652,6 +659,9 @@ gdata_youtube_query_set_language (GDataYouTubeQuery *self, const gchar *language
 	g_free (self->priv->language);
 	self->priv->language = g_strdup (language);
 	g_object_notify (G_OBJECT (self), "language");
+
+	/* Our current ETag will no longer be relevant */
+	gdata_query_set_etag (GDATA_QUERY (self), NULL);
 }
 
 /**
@@ -674,7 +684,7 @@ gdata_youtube_query_get_order_by (GDataYouTubeQuery *self)
 /**
  * gdata_youtube_query_set_order_by:
  * @self: a #GDataYouTubeQuery
- * @order_by: a new order by string, or %NULL
+ * @order_by: (allow-none): a new order by string, or %NULL
  *
  * Sets the #GDataYouTubeQuery:order-by property of the #GDataYouTubeQuery to the new order by string, @order_by.
  *
@@ -690,6 +700,9 @@ gdata_youtube_query_set_order_by (GDataYouTubeQuery *self, const gchar *order_by
 	g_free (self->priv->order_by);
 	self->priv->order_by = g_strdup (order_by);
 	g_object_notify (G_OBJECT (self), "order-by");
+
+	/* Our current ETag will no longer be relevant */
+	gdata_query_set_etag (GDATA_QUERY (self), NULL);
 }
 
 /**
@@ -712,7 +725,7 @@ gdata_youtube_query_get_restriction (GDataYouTubeQuery *self)
 /**
  * gdata_youtube_query_set_restriction:
  * @self: a #GDataYouTubeQuery
- * @restriction: a new restriction string, or %NULL
+ * @restriction: (allow-none): a new restriction string, or %NULL
  *
  * Sets the #GDataYouTubeQuery:restriction property of the #GDataYouTubeQuery to the new restriction string, @restriction.
  *
@@ -727,6 +740,9 @@ gdata_youtube_query_set_restriction (GDataYouTubeQuery *self, const gchar *restr
 	g_free (self->priv->restriction);
 	self->priv->restriction = g_strdup (restriction);
 	g_object_notify (G_OBJECT (self), "restriction");
+
+	/* Our current ETag will no longer be relevant */
+	gdata_query_set_etag (GDATA_QUERY (self), NULL);
 }
 
 /**
@@ -761,6 +777,9 @@ gdata_youtube_query_set_safe_search (GDataYouTubeQuery *self, GDataYouTubeSafeSe
 	g_return_if_fail (GDATA_IS_YOUTUBE_QUERY (self));
 	self->priv->safe_search = safe_search;
 	g_object_notify (G_OBJECT (self), "safe-search");
+
+	/* Our current ETag will no longer be relevant */
+	gdata_query_set_etag (GDATA_QUERY (self), NULL);
 }
 
 /**
@@ -797,6 +816,9 @@ gdata_youtube_query_set_sort_order (GDataYouTubeQuery *self, GDataYouTubeSortOrd
 	g_return_if_fail (GDATA_IS_YOUTUBE_QUERY (self));
 	self->priv->sort_order = sort_order;
 	g_object_notify (G_OBJECT (self), "sort-order");
+
+	/* Our current ETag will no longer be relevant */
+	gdata_query_set_etag (GDATA_QUERY (self), NULL);
 }
 
 /**
@@ -831,6 +853,9 @@ gdata_youtube_query_set_age (GDataYouTubeQuery *self, GDataYouTubeAge age)
 	g_return_if_fail (GDATA_IS_YOUTUBE_QUERY (self));
 	self->priv->age = age;
 	g_object_notify (G_OBJECT (self), "age");
+
+	/* Our current ETag will no longer be relevant */
+	gdata_query_set_etag (GDATA_QUERY (self), NULL);
 }
 
 /**
@@ -865,4 +890,7 @@ gdata_youtube_query_set_uploader (GDataYouTubeQuery *self, GDataYouTubeUploader
 	g_return_if_fail (GDATA_IS_YOUTUBE_QUERY (self));
 	self->priv->uploader = uploader;
 	g_object_notify (G_OBJECT (self), "uploader");
+
+	/* Our current ETag will no longer be relevant */
+	gdata_query_set_etag (GDATA_QUERY (self), NULL);
 }
