@@ -24,7 +24,6 @@
 #include "common.h"
 
 #define DEVELOPER_KEY "AI39si7Me3Q7zYs6hmkFvpRBD2nrkVjYYsUO5lh_3HdOkGRc9g6Z4nzxZatk_aAo2EsA21k7vrda0OO6oFg2rnhMedZXPyXoEw"
-#define YT_USERNAME "GDataTest"
 
 static void
 test_authentication (void)
@@ -51,7 +50,6 @@ test_authentication (void)
 	g_assert (gdata_service_is_authenticated (service) == TRUE);
 	g_assert_cmpstr (gdata_service_get_username (service), ==, USERNAME);
 	g_assert_cmpstr (gdata_service_get_password (service), ==, PASSWORD);
-	g_assert_cmpstr (gdata_youtube_service_get_youtube_user (GDATA_YOUTUBE_SERVICE (service)), ==, YT_USERNAME);
 
 	g_object_unref (service);
 }
@@ -73,7 +71,6 @@ test_authentication_async_cb (GDataService *service, GAsyncResult *async_result,
 	g_assert (gdata_service_is_authenticated (service) == TRUE);
 	g_assert_cmpstr (gdata_service_get_username (service), ==, USERNAME);
 	g_assert_cmpstr (gdata_service_get_password (service), ==, PASSWORD);
-	g_assert_cmpstr (gdata_youtube_service_get_youtube_user (GDATA_YOUTUBE_SERVICE (service)), ==, YT_USERNAME);
 }
 
 static void
@@ -261,62 +258,186 @@ test_query_related_async (gconstpointer service)
 	g_main_loop_unref (main_loop);
 }
 
-static void
-test_upload_simple (gconstpointer service)
-{
-	GDataYouTubeVideo *video, *new_video;
-	GDataMediaCategory *category;
+typedef struct {
+	GDataYouTubeService *service;
+	GDataYouTubeVideo *video;
+	GDataYouTubeVideo *updated_video;
 	GFile *video_file;
-	gchar *xml;
+	gchar *slug;
+	gchar *content_type;
+	GFileInputStream *file_stream;
+} UploadData;
+
+static void
+setup_upload (UploadData *data, gconstpointer service)
+{
+	GDataMediaCategory *category;
+	GFileInfo *file_info;
 	const gchar * const tags[] = { "toast", "wedding", NULL };
 	GError *error = NULL;
 
-	video = gdata_youtube_video_new (NULL);
+	data->service = g_object_ref ((gpointer) service);
 
-	gdata_entry_set_title (GDATA_ENTRY (video), "Bad Wedding Toast");
-	gdata_youtube_video_set_description (video, "I gave a bad toast at my friend's wedding.");
+	/* Create the metadata for the video being uploaded */
+	data->video = gdata_youtube_video_new (NULL);
+
+	gdata_entry_set_title (GDATA_ENTRY (data->video), "Bad Wedding Toast");
+	gdata_youtube_video_set_description (data->video, "I gave a bad toast at my friend's wedding.");
 	category = gdata_media_category_new ("People", "http://gdata.youtube.com/schemas/2007/categories.cat", NULL);
-	gdata_youtube_video_set_category (video, category);
+	gdata_youtube_video_set_category (data->video, category);
 	g_object_unref (category);
-	gdata_youtube_video_set_keywords (video, tags);
-
-	/* Check the XML */
-	xml = gdata_parsable_get_xml (GDATA_PARSABLE (video));
-	g_assert_cmpstr (xml, ==,
-			 "<?xml version='1.0' encoding='UTF-8'?>"
-			 "<entry xmlns='http://www.w3.org/2005/Atom' "
-				"xmlns:media='http://search.yahoo.com/mrss/' "
-				"xmlns:gd='http://schemas.google.com/g/2005' "
-				"xmlns:yt='http://gdata.youtube.com/schemas/2007' "
-				"xmlns:app='http://www.w3.org/2007/app'>"
-				"<title type='text'>Bad Wedding Toast</title>"
-				"<category term='http://gdata.youtube.com/schemas/2007#video' scheme='http://schemas.google.com/g/2005#kind'/>"
-				"<media:group>"
-					"<media:category scheme='http://gdata.youtube.com/schemas/2007/categories.cat'>People</media:category>"
-					"<media:title type='plain'>Bad Wedding Toast</media:title>"
-					"<media:description type='plain'>I gave a bad toast at my friend&apos;s wedding.</media:description>"
-					"<media:keywords>toast,wedding</media:keywords>"
-				"</media:group>"
-				"<app:control>"
-					"<app:draft>no</app:draft>"
-				"</app:control>"
-			 "</entry>");
-	g_free (xml);
+	gdata_youtube_video_set_keywords (data->video, tags);
 
+	/* Get a file to upload */
 	/* TODO: fix the path */
-	video_file = g_file_new_for_path (TEST_FILE_DIR "sample.ogg");
+	data->video_file = g_file_new_for_path (TEST_FILE_DIR "sample.ogg");
+
+	/* Get the file's info */
+	file_info = g_file_query_info (data->video_file, G_FILE_ATTRIBUTE_STANDARD_DISPLAY_NAME "," G_FILE_ATTRIBUTE_STANDARD_CONTENT_TYPE,
+	                               G_FILE_QUERY_INFO_NONE, NULL, &error);
+	g_assert_no_error (error);
+	g_assert (G_IS_FILE_INFO (file_info));
+
+	data->slug = g_strdup (g_file_info_get_display_name (file_info));
+	data->content_type = g_strdup (g_file_info_get_content_type (file_info));
+
+	g_object_unref (file_info);
+
+	/* Get an input stream for the file */
+	data->file_stream = g_file_read (data->video_file, NULL, &error);
+	g_assert_no_error (error);
+	g_assert (G_IS_FILE_INPUT_STREAM (data->file_stream));
+}
+
+static void
+teardown_upload (UploadData *data, gconstpointer service)
+{
+	/* Delete the uploaded video, if possible */
+	if (data->updated_video != NULL) {
+		gdata_service_delete_entry (GDATA_SERVICE (service), GDATA_ENTRY (data->updated_video), NULL, NULL);
+		g_object_unref (data->updated_video);
+	}
+
+	g_object_unref (data->video);
+	g_object_unref (data->video_file);
+	g_free (data->slug);
+	g_free (data->content_type);
+	g_object_unref (data->file_stream);
+	g_object_unref (data->service);
+}
+
+static void
+test_upload_simple (UploadData *data, gconstpointer service)
+{
+	GDataUploadStream *upload_stream;
+	const gchar * const *tags, * const *tags2;
+	gssize transfer_size;
+	GError *error = NULL;
+
+	/* Prepare the upload stream */
+	upload_stream = gdata_youtube_service_upload_video (GDATA_YOUTUBE_SERVICE (service), data->video, data->slug, data->content_type, NULL,
+	                                                    &error);
+	g_assert_no_error (error);
+	g_assert (GDATA_IS_UPLOAD_STREAM (upload_stream));
 
 	/* Upload the video */
-	new_video = gdata_youtube_service_upload_video (GDATA_YOUTUBE_SERVICE (service), video, video_file, NULL, &error);
+	transfer_size = g_output_stream_splice (G_OUTPUT_STREAM (upload_stream), G_INPUT_STREAM (data->file_stream),
+	                                        G_OUTPUT_STREAM_SPLICE_CLOSE_SOURCE | G_OUTPUT_STREAM_SPLICE_CLOSE_TARGET, NULL, &error);
 	g_assert_no_error (error);
-	g_assert (GDATA_IS_YOUTUBE_VIDEO (new_video));
-	g_clear_error (&error);
+	g_assert_cmpint (transfer_size, >, 0);
 
-	/* TODO: check entries and feed properties */
+	/* Finish off the upload */
+	data->updated_video = gdata_youtube_service_finish_video_upload (GDATA_YOUTUBE_SERVICE (service), upload_stream, &error);
+	g_assert_no_error (error);
+	g_assert (GDATA_IS_YOUTUBE_VIDEO (data->updated_video));
 
-	g_object_unref (video);
-	g_object_unref (new_video);
-	g_object_unref (video_file);
+	/* Check the video's properties */
+	g_assert (gdata_entry_is_inserted (GDATA_ENTRY (data->updated_video)));
+	g_assert_cmpstr (gdata_entry_get_title (GDATA_ENTRY (data->updated_video)), ==, gdata_entry_get_title (GDATA_ENTRY (data->video)));
+	g_assert_cmpstr (gdata_youtube_video_get_description (data->updated_video), ==, gdata_youtube_video_get_description (data->video));
+	g_assert_cmpstr (gdata_media_category_get_category (gdata_youtube_video_get_category (data->updated_video)), ==,
+	                 gdata_media_category_get_category (gdata_youtube_video_get_category (data->video)));
+
+	tags = gdata_youtube_video_get_keywords (data->video);
+	tags2 = gdata_youtube_video_get_keywords (data->updated_video);
+	g_assert_cmpuint (g_strv_length ((gchar**) tags2), ==, g_strv_length ((gchar**) tags));
+	g_assert_cmpstr (tags2[0], ==, tags[0]);
+	g_assert_cmpstr (tags2[1], ==, tags[1]);
+	g_assert_cmpstr (tags2[2], ==, tags[2]);
+}
+
+typedef struct {
+	UploadData data;
+	GMainLoop *main_loop;
+} UploadAsyncData;
+
+static void
+setup_upload_async (UploadAsyncData *data, gconstpointer service)
+{
+	setup_upload ((UploadData*) data, service);
+	data->main_loop = g_main_loop_new (NULL, TRUE);
+}
+
+static void
+teardown_upload_async (UploadAsyncData *data, gconstpointer service)
+{
+	teardown_upload ((UploadData*) data, service);
+	g_main_loop_unref (data->main_loop);
+}
+
+static void
+test_upload_async_cb (GOutputStream *stream, GAsyncResult *result, UploadAsyncData *data)
+{
+	const gchar * const *tags, * const *tags2;
+	gssize transfer_size;
+	GError *error = NULL;
+
+	/* Finish off the transfer */
+	transfer_size = g_output_stream_splice_finish (stream, result, &error);
+	g_assert_no_error (error);
+	g_assert_cmpint (transfer_size, >, 0);
+
+	/* Finish off the upload */
+	data->data.updated_video = gdata_youtube_service_finish_video_upload (data->data.service, GDATA_UPLOAD_STREAM (stream), &error);
+	g_assert_no_error (error);
+	g_assert (GDATA_IS_YOUTUBE_VIDEO (data->data.updated_video));
+
+	/* Check the video's properties */
+	g_assert (gdata_entry_is_inserted (GDATA_ENTRY (data->data.updated_video)));
+	g_assert_cmpstr (gdata_entry_get_title (GDATA_ENTRY (data->data.updated_video)), ==, gdata_entry_get_title (GDATA_ENTRY (data->data.video)));
+	g_assert_cmpstr (gdata_youtube_video_get_description (data->data.updated_video), ==, gdata_youtube_video_get_description (data->data.video));
+	g_assert_cmpstr (gdata_media_category_get_category (gdata_youtube_video_get_category (data->data.updated_video)), ==,
+	                 gdata_media_category_get_category (gdata_youtube_video_get_category (data->data.video)));
+
+	tags = gdata_youtube_video_get_keywords (data->data.video);
+	tags2 = gdata_youtube_video_get_keywords (data->data.updated_video);
+	g_assert_cmpuint (g_strv_length ((gchar**) tags2), ==, g_strv_length ((gchar**) tags));
+	g_assert_cmpstr (tags2[0], ==, tags[0]);
+	g_assert_cmpstr (tags2[1], ==, tags[1]);
+	g_assert_cmpstr (tags2[2], ==, tags[2]);
+
+	g_main_loop_quit (data->main_loop);
+}
+
+static void
+test_upload_async (UploadAsyncData *data, gconstpointer service)
+{
+	GDataUploadStream *upload_stream;
+	GError *error = NULL;
+
+	/* Prepare the upload stream */
+	upload_stream = gdata_youtube_service_upload_video (GDATA_YOUTUBE_SERVICE (service), data->data.video, data->data.slug,
+	                                                    data->data.content_type, NULL, &error);
+	g_assert_no_error (error);
+	g_assert (GDATA_IS_UPLOAD_STREAM (upload_stream));
+
+	/* Upload the video asynchronously */
+	g_output_stream_splice_async (G_OUTPUT_STREAM (upload_stream), G_INPUT_STREAM (data->data.file_stream),
+	                              G_OUTPUT_STREAM_SPLICE_CLOSE_SOURCE | G_OUTPUT_STREAM_SPLICE_CLOSE_TARGET, G_PRIORITY_DEFAULT, NULL,
+	                              (GAsyncReadyCallback) test_upload_async_cb, data);
+	g_main_loop_run (data->main_loop);
+
+	g_object_unref (upload_stream);
 }
 
 static void
@@ -376,7 +497,6 @@ test_parsing_yt_recorded (void)
 {
 	GDataYouTubeVideo *video;
 	gint64 recorded;
-	gchar *xml;
 	GError *error = NULL;
 
 	video = GDATA_YOUTUBE_VIDEO (gdata_parsable_new_from_xml (GDATA_TYPE_YOUTUBE_VIDEO,
@@ -416,14 +536,15 @@ test_parsing_yt_recorded (void)
 	gdata_youtube_video_set_recorded (video, recorded);
 
 	/* Check the XML */
-	xml = gdata_parsable_get_xml (GDATA_PARSABLE (video));
-	g_assert_cmpstr (xml, ==,
+	gdata_test_assert_xml (video,
 			 "<?xml version='1.0' encoding='UTF-8'?>"
 			 "<entry xmlns='http://www.w3.org/2005/Atom' "
 				"xmlns:media='http://search.yahoo.com/mrss/' "
 				"xmlns:gd='http://schemas.google.com/g/2005' "
 				"xmlns:yt='http://gdata.youtube.com/schemas/2007' "
 				"xmlns:app='http://www.w3.org/2007/app' "
+				"xmlns:georss='http://www.georss.org/georss' "
+				"xmlns:gml='http://www.opengis.net/gml' "
 				"gd:etag='W/\"CEMFSX47eCp7ImA9WxVUGEw.\"'>"
 				"<title type='text'>Judas Priest - Painkiller</title>"
 				"<id>tag:youtube.com,2008:video:JAagedeKdcQ</id>"
@@ -445,7 +566,6 @@ test_parsing_yt_recorded (void)
 					"<app:draft>no</app:draft>"
 				"</app:control>"
 			 "</entry>");
-	g_free (xml);
 
 	/* TODO: more tests on entry properties */
 
@@ -456,7 +576,6 @@ static void
 test_parsing_yt_access_control (void)
 {
 	GDataYouTubeVideo *video;
-	gchar *xml;
 	GError *error = NULL;
 
 	video = GDATA_YOUTUBE_VIDEO (gdata_parsable_new_from_xml (GDATA_TYPE_YOUTUBE_VIDEO,
@@ -505,14 +624,15 @@ test_parsing_yt_access_control (void)
 	gdata_youtube_video_set_access_control (video, GDATA_YOUTUBE_ACTION_EMBED, GDATA_YOUTUBE_PERMISSION_DENIED);
 
 	/* Check the XML */
-	xml = gdata_parsable_get_xml (GDATA_PARSABLE (video));
-	g_assert_cmpstr (xml, ==,
+	gdata_test_assert_xml (video,
 			 "<?xml version='1.0' encoding='UTF-8'?>"
 			 "<entry xmlns='http://www.w3.org/2005/Atom' "
 				"xmlns:media='http://search.yahoo.com/mrss/' "
 				"xmlns:gd='http://schemas.google.com/g/2005' "
 				"xmlns:yt='http://gdata.youtube.com/schemas/2007' "
 				"xmlns:app='http://www.w3.org/2007/app' "
+				"xmlns:georss='http://www.georss.org/georss' "
+				"xmlns:gml='http://www.opengis.net/gml' "
 				"gd:etag='W/\"CEMFSX47eCp7ImA9WxVUGEw.\"'>"
 				"<title type='text'>Judas Priest - Painkiller</title>"
 				"<id>tag:youtube.com,2008:video:JAagedeKdcQ</id>"
@@ -540,7 +660,6 @@ test_parsing_yt_access_control (void)
 					"<app:draft>no</app:draft>"
 				"</app:control>"
 			 "</entry>");
-	g_free (xml);
 
 	g_object_unref (video);
 }
@@ -649,6 +768,226 @@ test_parsing_comments_feed_link (void)
 	g_object_unref (video);
 }*/
 
+static void
+test_parsing_georss_where (void)
+{
+	GDataYouTubeVideo *video;
+	gdouble latitude, longitude;
+	GError *error = NULL;
+
+	video = GDATA_YOUTUBE_VIDEO (gdata_parsable_new_from_xml (GDATA_TYPE_YOUTUBE_VIDEO,
+		"<entry xmlns='http://www.w3.org/2005/Atom' "
+		       "xmlns:media='http://search.yahoo.com/mrss/' "
+		       "xmlns:yt='http://gdata.youtube.com/schemas/2007' "
+		       "xmlns:gd='http://schemas.google.com/g/2005' "
+		       "xmlns:georss='http://www.georss.org/georss' "
+		       "xmlns:gml='http://www.opengis.net/gml'>"
+			"<id>tag:youtube.com,2008:video:JAagedeKdcQ</id>"
+			"<published>2006-05-16T14:06:37.000Z</published>"
+			"<updated>2009-03-23T12:46:58.000Z</updated>"
+			"<category scheme='http://schemas.google.com/g/2005#kind' term='http://gdata.youtube.com/schemas/2007#video'/>"
+			"<title>Some video somewhere</title>"
+			"<link rel='http://www.iana.org/assignments/relation/alternate' type='text/html' href='http://www.youtube.com/watch?v=JAagedeKdcQ'/>"
+			"<link rel='http://www.iana.org/assignments/relation/self' type='application/atom+xml' href='http://gdata.youtube.com/feeds/api/videos/JAagedeKdcQ?client=ytapi-google-jsdemo'/>"
+			"<author>"
+				"<name>Foo</name>"
+				"<uri>http://gdata.youtube.com/feeds/api/users/Foo</uri>"
+			"</author>"
+			"<media:group>"
+				"<media:title type='plain'>Some video somewhere</media:title>"
+				"<media:credit role='uploader' scheme='urn:youtube'>Foo</media:credit>"
+				"<media:category label='Music' scheme='http://gdata.youtube.com/schemas/2007/categories.cat'>Music</media:category>"
+			"</media:group>"
+			"<georss:where>"
+				"<gml:Point>"
+					"<gml:pos>41.14556884765625 -8.63525390625</gml:pos>"
+				"</gml:Point>"
+			"</georss:where>"
+		"</entry>", -1, &error));
+	g_assert_no_error (error);
+	g_assert (GDATA_IS_YOUTUBE_VIDEO (video));
+	g_clear_error (&error);
+
+	/* Test the coordinates */
+	gdata_youtube_video_get_coordinates (video, &latitude, &longitude);
+	g_assert_cmpfloat (latitude, ==, 41.14556884765625);
+	g_assert_cmpfloat (longitude, ==, -8.63525390625);
+
+	/* Update them and see if they're set OK and the XML's written out OK */
+	gdata_youtube_video_set_coordinates (video, 5.5, 6.5);
+
+	g_object_get (G_OBJECT (video),
+	              "latitude", &latitude,
+	              "longitude", &longitude,
+	              NULL);
+
+	g_assert_cmpfloat (latitude, ==, 5.5);
+	g_assert_cmpfloat (longitude, ==, 6.5);
+
+	/* Check the XML */
+	gdata_test_assert_xml (video,
+		"<?xml version='1.0' encoding='UTF-8'?>"
+		"<entry xmlns='http://www.w3.org/2005/Atom' "
+		       "xmlns:app='http://www.w3.org/2007/app' "
+		       "xmlns:media='http://search.yahoo.com/mrss/' "
+		       "xmlns:yt='http://gdata.youtube.com/schemas/2007' "
+		       "xmlns:gd='http://schemas.google.com/g/2005' "
+		       "xmlns:georss='http://www.georss.org/georss' "
+		       "xmlns:gml='http://www.opengis.net/gml'>"
+			"<title type='text'>Some video somewhere</title>"
+			"<id>tag:youtube.com,2008:video:JAagedeKdcQ</id>"
+			"<updated>2009-03-23T12:46:58Z</updated>"
+			"<published>2006-05-16T14:06:37Z</published>"
+			"<category term='http://gdata.youtube.com/schemas/2007#video' scheme='http://schemas.google.com/g/2005#kind'/>"
+			"<link href='http://www.youtube.com/watch?v=JAagedeKdcQ' rel='http://www.iana.org/assignments/relation/alternate' type='text/html'/>"
+			"<link href='http://gdata.youtube.com/feeds/api/videos/JAagedeKdcQ?client=ytapi-google-jsdemo' rel='http://www.iana.org/assignments/relation/self' type='application/atom+xml'/>"
+			"<author>"
+				"<name>Foo</name>"
+				"<uri>http://gdata.youtube.com/feeds/api/users/Foo</uri>"
+			"</author>"
+			"<media:group>"
+				"<media:category scheme='http://gdata.youtube.com/schemas/2007/categories.cat' label='Music'>Music</media:category>"
+				"<media:title type='plain'>Some video somewhere</media:title>"
+			"</media:group>"
+			"<app:control><app:draft>no</app:draft></app:control>"
+			"<georss:where>"
+				"<gml:Point>"
+					"<gml:pos>5.5 6.5</gml:pos>"
+				"</gml:Point>"
+			"</georss:where>"
+		"</entry>");
+
+	/* Unset the properties and ensure they're removed from the XML */
+	gdata_youtube_video_set_coordinates (video, G_MAXDOUBLE, G_MAXDOUBLE);
+
+	gdata_youtube_video_get_coordinates (video, &latitude, &longitude);
+	g_assert_cmpfloat (latitude, ==, G_MAXDOUBLE);
+	g_assert_cmpfloat (longitude, ==, G_MAXDOUBLE);
+
+	/* Check the XML */
+	gdata_test_assert_xml (video,
+		"<?xml version='1.0' encoding='UTF-8'?>"
+		"<entry xmlns='http://www.w3.org/2005/Atom' "
+		       "xmlns:app='http://www.w3.org/2007/app' "
+		       "xmlns:media='http://search.yahoo.com/mrss/' "
+		       "xmlns:yt='http://gdata.youtube.com/schemas/2007' "
+		       "xmlns:gd='http://schemas.google.com/g/2005' "
+		       "xmlns:georss='http://www.georss.org/georss' "
+		       "xmlns:gml='http://www.opengis.net/gml'>"
+			"<title type='text'>Some video somewhere</title>"
+			"<id>tag:youtube.com,2008:video:JAagedeKdcQ</id>"
+			"<updated>2009-03-23T12:46:58Z</updated>"
+			"<published>2006-05-16T14:06:37Z</published>"
+			"<category term='http://gdata.youtube.com/schemas/2007#video' scheme='http://schemas.google.com/g/2005#kind'/>"
+			"<link href='http://www.youtube.com/watch?v=JAagedeKdcQ' rel='http://www.iana.org/assignments/relation/alternate' type='text/html'/>"
+			"<link href='http://gdata.youtube.com/feeds/api/videos/JAagedeKdcQ?client=ytapi-google-jsdemo' rel='http://www.iana.org/assignments/relation/self' type='application/atom+xml'/>"
+			"<author>"
+				"<name>Foo</name>"
+				"<uri>http://gdata.youtube.com/feeds/api/users/Foo</uri>"
+			"</author>"
+			"<media:group>"
+				"<media:category scheme='http://gdata.youtube.com/schemas/2007/categories.cat' label='Music'>Music</media:category>"
+				"<media:title type='plain'>Some video somewhere</media:title>"
+			"</media:group>"
+			"<app:control><app:draft>no</app:draft></app:control>"
+		"</entry>");
+
+	g_object_unref (video);
+}
+
+static void
+test_parsing_media_group (void)
+{
+	GDataYouTubeVideo *video;
+	GError *error = NULL;
+
+	video = GDATA_YOUTUBE_VIDEO (gdata_parsable_new_from_xml (GDATA_TYPE_YOUTUBE_VIDEO,
+		"<entry xmlns='http://www.w3.org/2005/Atom' "
+		       "xmlns:media='http://search.yahoo.com/mrss/' "
+		       "xmlns:yt='http://gdata.youtube.com/schemas/2007' "
+		       "xmlns:gd='http://schemas.google.com/g/2005'>"
+			"<id>tag:youtube.com,2008:video:JAagedeKdcQ</id>"
+			"<published>2006-05-16T14:06:37.000Z</published>"
+			"<updated>2009-03-23T12:46:58.000Z</updated>"
+			"<category scheme='http://schemas.google.com/g/2005#kind' term='http://gdata.youtube.com/schemas/2007#video'/>"
+			"<title>Some video somewhere</title>"
+			"<link rel='http://www.iana.org/assignments/relation/alternate' type='text/html' href='http://www.youtube.com/watch?v=JAagedeKdcQ'/>"
+			"<link rel='http://www.iana.org/assignments/relation/self' type='application/atom+xml' href='http://gdata.youtube.com/feeds/api/videos/JAagedeKdcQ?client=ytapi-google-jsdemo'/>"
+			"<author>"
+				"<name>Foo</name>"
+				"<uri>http://gdata.youtube.com/feeds/api/users/Foo</uri>"
+			"</author>"
+			"<media:group>"
+				"<media:category label='Shows' scheme='http://gdata.youtube.com/schemas/2007/categories.cat'>Shows</media:category>"
+				"<media:category scheme='http://gdata.youtube.com/schemas/2007/releasemediums.cat'>6</media:category>"
+				"<media:category scheme='http://gdata.youtube.com/schemas/2007/mediatypes.cat'>3</media:category>"
+				"<media:content url='http://www.youtube.com/v/aklRlKH4R94?f=related&amp;d=ARK7_SyB_5iKQvGvwsk-0D4O88HsQjpE1a8d1GxQnGDm&amp;app=youtube_gdata' type='application/x-shockwave-flash' medium='video' isDefault='true' expression='full' duration='163' yt:format='5'/>"
+				"<media:content url='rtsp://v3.cache6.c.youtube.com/CkYLENy73wIaPQneR_ihlFFJahMYDSANFEgGUgdyZWxhdGVkciEBErv9LIH_mIpC8a_CyT7QPg7zwexCOkTVrx3UbFCcYOYM/0/0/0/video.3gp' type='video/3gpp' medium='video' expression='full' duration='163' yt:format='1'/>"
+				"<media:content url='rtsp://v3.cache3.c.youtube.com/CkYLENy73wIaPQneR_ihlFFJahMYESARFEgGUgdyZWxhdGVkciEBErv9LIH_mIpC8a_CyT7QPg7zwexCOkTVrx3UbFCcYOYM/0/0/0/video.3gp' type='video/3gpp' medium='video' expression='full' duration='163' yt:format='6'/>"
+				"<media:credit role='uploader' scheme='urn:youtube' yt:type='partner'>machinima</media:credit>"
+				"<media:credit role='Producer' scheme='urn:ebu'>Machinima</media:credit>"
+				"<media:credit role='info' scheme='urn:ebu'>season 1 episode 4 air date 08/22/10</media:credit>"
+				"<media:credit role='Producer' scheme='urn:ebu'>Machinima</media:credit>"
+				"<media:credit role='info' scheme='urn:ebu'>season 1 episode 4 air date 08/22/10</media:credit>"
+				"<media:description type='plain'>www.youtube.com Click here to watch If It Were Realistic: Melee If It Were Realistic: Gravity Gun (Half Life 2 Machinima) What if gravity guns were realistic? Created by Renaldoxx from Massive X Productions Directors Channel: www.youtube.com www.youtube.com - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - Follow Machinima on Twitter! Machinima twitter.com Inside Gaming twitter.com Machinima Respawn twitter.com Machinima Entertainment, Technology, Culture twitter.com FOR MORE MACHINIMA, GO TO: www.youtube.com FOR MORE GAMEPLAY, GO TO: www.youtube.com FOR MORE SPORTS GAMEPLAY, GO TO: www.youtube.com FOR MORE TRAILERS, GO TO: www.youtube.com</media:description>"
+				"<media:keywords>Half, Life, If, It, Were, Realistic, Gravity, Gun, Renaldoxx, Sniper, Game, Machinima, Action, Gordon, Freeman, drift0r, Euphorian, Films, Combine, Rebel, Dark, Citizen, Diary, massivex, Productions, Massive, yt:quality=high, Half-Life, [2], HL2, fortress, gmod, left dead, tf2</media:keywords>"
+				"<media:player url='http://www.youtube.com/watch?v=aklRlKH4R94&amp;feature=youtube_gdata_player'/>"
+				"<media:rating scheme='urn:mpaa'>pg</media:rating>"
+				"<media:thumbnail url='http://i.ytimg.com/vi/aklRlKH4R94/default.jpg' height='90' width='120' time='00:01:21.500' yt:name='default'/>"
+				"<media:thumbnail url='http://i.ytimg.com/vi/aklRlKH4R94/hqdefault.jpg' height='360' width='480' yt:name='hqdefault'/>"
+				"<media:thumbnail url='http://i.ytimg.com/vi/aklRlKH4R94/1.jpg' height='90' width='120' time='00:00:40.750' yt:name='start'/>"
+				"<media:thumbnail url='http://i.ytimg.com/vi/aklRlKH4R94/2.jpg' height='90' width='120' time='00:01:21.500' yt:name='middle'/>"
+				"<media:thumbnail url='http://i.ytimg.com/vi/aklRlKH4R94/3.jpg' height='90' width='120' time='00:02:02.250' yt:name='end'/>"
+				"<media:title type='plain'>If It Were Realistic - Gravity Gun (Half Life 2 Machinima)</media:title>"
+				"<yt:aspectRatio>widescreen</yt:aspectRatio>"
+				"<yt:duration seconds='163'/>"
+				"<yt:uploaded>2010-08-22T14:04:18.000Z</yt:uploaded>"
+				"<yt:videoid>aklRlKH4R94</yt:videoid>"
+			"</media:group>"
+		"</entry>", -1, &error));
+	g_assert_no_error (error);
+	g_assert (GDATA_IS_YOUTUBE_VIDEO (video));
+	g_clear_error (&error);
+
+	/* TODO: For the moment, we just check that parsing the XML didn't fail. Later, we might actually support outputting the XML again. */
+
+	g_object_unref (video);
+}
+
+static void
+test_video_escaping (void)
+{
+	GDataYouTubeVideo *video;
+	const gchar * const keywords[] = { "<keyword1>", "keyword2 & stuff, things", NULL };
+
+	video = gdata_youtube_video_new (NULL);
+	gdata_youtube_video_set_location (video, "Here & there");
+	gdata_youtube_video_set_access_control (video, "<action>", GDATA_YOUTUBE_PERMISSION_ALLOWED);
+	gdata_youtube_video_set_keywords (video, keywords);
+	gdata_youtube_video_set_description (video, "Description & stuff.");
+	gdata_youtube_video_set_aspect_ratio (video, "4 & 3");
+
+	/* Check the outputted XML is escaped properly */
+	gdata_test_assert_xml (video,
+	                 "<?xml version='1.0' encoding='UTF-8'?>"
+	                 "<entry xmlns='http://www.w3.org/2005/Atom' xmlns:media='http://search.yahoo.com/mrss/' "
+	                        "xmlns:gd='http://schemas.google.com/g/2005' "
+	                        "xmlns:yt='http://gdata.youtube.com/schemas/2007' xmlns:app='http://www.w3.org/2007/app' "
+	                        "xmlns:georss='http://www.georss.org/georss' xmlns:gml='http://www.opengis.net/gml'>"
+				"<title type='text'></title>"
+				"<category term='http://gdata.youtube.com/schemas/2007#video' scheme='http://schemas.google.com/g/2005#kind'/>"
+				"<media:group>"
+					"<media:description type='plain'>Description &amp; stuff.</media:description>"
+					"<media:keywords>&lt;keyword1&gt;,keyword2 &amp; stuff%2C things</media:keywords>"
+					"<yt:aspectratio>4 &amp; 3</yt:aspectratio>"
+				"</media:group>"
+				"<yt:location>Here &amp; there</yt:location>"
+				"<yt:accessControl action='&lt;action&gt;' permission='allowed'/>"
+				"<app:control><app:draft>no</app:draft></app:control>"
+	                 "</entry>");
+	g_object_unref (video);
+}
+
 static void
 test_query_uri (void)
 {
@@ -1012,13 +1351,11 @@ static void
 test_batch_async (BatchData *data, gconstpointer service)
 {
 	GDataBatchOperation *operation;
-	guint op_id;
 	GMainLoop *main_loop;
 
 	/* Run an async query operation on the video */
 	operation = gdata_batchable_create_operation (GDATA_BATCHABLE (service), "http://gdata.youtube.com/feeds/api/videos/batch");
-	op_id = gdata_test_batch_operation_query (operation, gdata_entry_get_id (data->new_video), GDATA_TYPE_YOUTUBE_VIDEO, data->new_video, NULL,
-	                                          NULL);
+	gdata_test_batch_operation_query (operation, gdata_entry_get_id (data->new_video), GDATA_TYPE_YOUTUBE_VIDEO, data->new_video, NULL, NULL);
 
 	main_loop = g_main_loop_new (NULL, TRUE);
 
@@ -1044,14 +1381,13 @@ static void
 test_batch_async_cancellation (BatchData *data, gconstpointer service)
 {
 	GDataBatchOperation *operation;
-	guint op_id;
 	GMainLoop *main_loop;
 	GCancellable *cancellable;
+	GError *error = NULL;
 
 	/* Run an async query operation on the video */
 	operation = gdata_batchable_create_operation (GDATA_BATCHABLE (service), "http://gdata.youtube.com/feeds/api/videos/batch");
-	op_id = gdata_test_batch_operation_query (operation, gdata_entry_get_id (data->new_video), GDATA_TYPE_YOUTUBE_VIDEO, data->new_video, NULL,
-	                                          NULL);
+	gdata_test_batch_operation_query (operation, gdata_entry_get_id (data->new_video), GDATA_TYPE_YOUTUBE_VIDEO, data->new_video, NULL, &error);
 
 	main_loop = g_main_loop_new (NULL, TRUE);
 	cancellable = g_cancellable_new ();
@@ -1060,8 +1396,13 @@ test_batch_async_cancellation (BatchData *data, gconstpointer service)
 	g_cancellable_cancel (cancellable); /* this should cancel the operation before it even starts, as we haven't run the main loop yet */
 
 	g_main_loop_run (main_loop);
+
+	g_assert_error (error, G_IO_ERROR, G_IO_ERROR_CANCELLED);
+	g_clear_error (&error);
+
 	g_main_loop_unref (main_loop);
 	g_object_unref (cancellable);
+	g_object_unref (operation);
 }
 
 static void
@@ -1091,7 +1432,8 @@ main (int argc, char *argv[])
 		g_test_add_data_func ("/youtube/query/related", service, test_query_related);
 		g_test_add_data_func ("/youtube/query/related_async", service, test_query_related_async);
 
-		g_test_add_data_func ("/youtube/upload/simple", service, test_upload_simple);
+		g_test_add ("/youtube/upload/simple", UploadData, service, setup_upload, test_upload_simple, teardown_upload);
+		g_test_add ("/youtube/upload/async", UploadAsyncData, service, setup_upload_async, test_upload_async, teardown_upload_async);
 
 		g_test_add_data_func ("/youtube/query/single", service, test_query_single);
 		g_test_add_data_func ("/youtube/query/single_async", service, test_query_single_async);
@@ -1110,6 +1452,10 @@ main (int argc, char *argv[])
 	g_test_add_func ("/youtube/parsing/yt:accessControl", test_parsing_yt_access_control);
 	g_test_add_func ("/youtube/parsing/yt:category", test_parsing_yt_category);
 	g_test_add_func ("/youtube/parsing/video_id_from_uri", test_parsing_video_id_from_uri);
+	g_test_add_func ("/youtube/parsing/georss:where", test_parsing_georss_where);
+	g_test_add_func ("/youtube/parsing/media:group", test_parsing_media_group);
+
+	g_test_add_func ("/youtube/video/escaping", test_video_escaping);
 
 	g_test_add_func ("/youtube/query/uri", test_query_uri);
 	g_test_add_func ("/youtube/query/etag", test_query_etag);
